@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: app.c,v 1.23 2003/11/24 22:10:40 dom Exp $
- *   $Date: 2003/11/24 22:10:40 $
+ *   $Id: app.c,v 1.29 2003/11/27 20:25:12 dom Exp $
+ *   $Date: 2003/11/27 20:25:12 $
  *
  *
  *   Implements the application
@@ -18,6 +18,16 @@
 /* Now get some message types */
 #include "msgs.h"
 
+#include <dirent.h>
+#include <sys/stat.h>
+
+
+
+typedef struct {
+    char   *name;
+    int     flags;
+} filelist_t;
+
 
 struct _app {
     dongle_t  *dongle;
@@ -28,6 +38,17 @@ struct _app {
     char      *next_media;
 };
 
+
+typedef struct {
+    app_t      *app;
+    char       *path;
+    /* For handling scanning of directories */
+    int         filelist_num;
+    filelist_t *filelist;
+} dirlist_t;
+
+
+
 typedef struct {
     char     *name;
     char     *url;
@@ -46,23 +67,36 @@ static int           liveradio_num    = 0;
 static channel_t    *liveradio        = NULL;
 static time_t        liveradio_time   = 0;
 
-static int           app_main(menu_t *menu,app_t *app, int sel);
-static int           app_live_tv(menu_t *menu,app_t *app, int sel);
-static int           process_livetv(menu_t *menu,app_t *app, int sel);
+static int           app_main(menu_t *menu,void *iapp, int sel);
+static int           app_live_tv(menu_t *menu,void *iapp, int sel);
+static int           process_livetv(menu_t *menu,void *iapp, int sel);
 static int           livetv_keys(void *ptr, int code);
 static int           livetv_ack(int acktype, void *param, unsigned char *buf, int len);
 
-static int           app_live_radio(menu_t *menu,app_t *app, int sel);
-static int           process_liveradio(menu_t *menu,app_t *app, int sel);
+static int           app_live_radio(menu_t *menu,void *iapp, int sel);
+static int           process_liveradio(menu_t *menu,void *iapp, int sel);
 static int           liveradio_keys(void *ptr, int code);
 static int           liveradio_ack(int acktype, void *param, unsigned char *buf, int len);
 
 
-static int           app_recordings(menu_t *menu,app_t *app, int sel);
-static int           app_music(menu_t *menu,app_t *app, int sel);
+static int           app_recordings(menu_t *menu,void *iapp, int sel);
+
+
+static int           app_music(menu_t *menu,void *iapp, int sel);
+static int           process_music(menu_t *menu,void *iapp, int sel);
+static int           music_keys(void *ptr, int code);
+static int           music_ack(int acktype, void *param, unsigned char *buf, int len);
+
+static int           app_settings(menu_t *menu,void *iapp, int sel);
+static int           process_settings(menu_t *menu,void *iapp, int sel);
+
+static filelist_t   *directory_scan(char *path, int *scanned);
+static void          directory_sort(int num,filelist_t *list);
+
+static dirlist_t    *new_dirlist(app_t *app);
+static void          delete_dirlist(void *dir);    /* Ahem */
+
 
-static int           app_settings(menu_t *menu,app_t *app, int sel);
-static int           process_settings(menu_t *menu,app_t *app, int sel);
 
 
 
@@ -116,13 +150,13 @@ void delete_app(app_t *app)
     delete_menu_tree(app->menu);
 
     if ( app->current_media ) {
-	free(app->current_media);
-	app->current_media = NULL;
+        free(app->current_media);
+        app->current_media = NULL;
     }
 
     if ( app->next_media ) {
-	free(app->next_media);
-	app->next_media = NULL;
+        free(app->next_media);
+        app->next_media = NULL;
     }
 
     free(app);
@@ -131,8 +165,10 @@ void delete_app(app_t *app)
 
 
 
-static int app_main(menu_t *menu,app_t *app, int sel)
+static int app_main(menu_t *menu,void *iapp, int sel)
 {   
+    app_t   *app = iapp;
+
     app->menu = new_menu(app->dongle,app->render,NULL,app,MENU_NUMBERS);
     menu_set_title(app->menu,"MediaMVP");
     menu_add(app->menu,"Live TV",app_live_tv);
@@ -151,8 +187,9 @@ static int app_main(menu_t *menu,app_t *app, int sel)
  *
  ***********************/
 
-static int app_live_tv(menu_t *menu,app_t *app, int sel)
+static int app_live_tv(menu_t *menu,void *iapp, int sel)
 {
+    app_t     *app = iapp;
     channel_t *chan;
     menu_t    *child;
     int        i;
@@ -171,8 +208,10 @@ static int app_live_tv(menu_t *menu,app_t *app, int sel)
     return 1;
 }
 
-static int process_livetv(menu_t *menu,app_t *app, int sel)
+static int process_livetv(menu_t *menu,void *iapp, int sel)
 {
+    app_t     *app = iapp;
+
     channel_t *chan = &livetv[sel];
     gui_send_play(app->gui,chan->url);
     gui_register_keys(app->gui,REGISTER_MEDIA,livetv_keys,app);
@@ -195,25 +234,25 @@ static int livetv_ack(int acktype, void *param, unsigned char *buf, int len)
         /* The client has sent a stop message, this could be response to "Stop" key press or
            us sending out a stop so we can switch to the net media file */ 
         if ( app->next_media ) {
-	    if ( app->current_media ) {
-		free(app->current_media);
-		app->current_media = NULL;
-	    }
+            if ( app->current_media ) {
+                free(app->current_media);
+                app->current_media = NULL;
+            }
             gui_send_play(app->gui,app->next_media);
-	    app->current_media = app->next_media;
+            app->current_media = app->next_media;
             free(app->next_media);
         } else {
-	    if ( ( stream_get_lasttype(app->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
-		gui_send_message(app->gui,&rfb_media_menu);
-		gui_send_message(app->gui,&rfb_display_on);
-	    }
-	    if ( app->current_media ) {
-		free(app->current_media);
-		app->current_media = NULL;
-	    }
-	    /* Remove registered handlers */
-	    gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
-	    gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
+            if ( ( stream_get_lasttype(app->dongle->stream) & MEDIA_MASK ) == MEDIA_MPEG ) {
+                gui_send_message(app->gui,&rfb_media_menu);
+                gui_send_message(app->gui,&rfb_display_on);
+            }
+            if ( app->current_media ) {
+                free(app->current_media);
+                app->current_media = NULL;
+            }
+            /* Remove registered handlers */
+            gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
+            gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
         }
 
 
@@ -246,7 +285,7 @@ static int livetv_keys(void *ptr, int code)
         gui_send_message(app->gui,&rfb_media_mute);
         break;
     default:
-	return 0;  /* Do more keys */
+        return 0;  /* Do more keys */
     }
     return 1;
 }
@@ -254,12 +293,13 @@ static int livetv_keys(void *ptr, int code)
 
 /***********************
  *
- *   Handling of the Live TV Menus
+ *   Handling of the Live Radio Menus
  *
  ***********************/
 
-static int app_live_radio(menu_t *menu,app_t *app, int sel)
+static int app_live_radio(menu_t *menu,void *iapp, int sel)
 {
+    app_t     *app = iapp;
     channel_t *chan;
     menu_t    *child;
     int        i;
@@ -276,19 +316,21 @@ static int app_live_radio(menu_t *menu,app_t *app, int sel)
     return 1;
 }
 
-static int process_liveradio(menu_t *menu,app_t *app, int sel)
+static int process_liveradio(menu_t *menu,void *iapp, int sel)
 {
+    app_t     *app = iapp;
     channel_t *chan = &liveradio[sel];
+
     printf("Sending media do request %s\n",chan->url);
-    if ( app->current_media == NULL ) {	
-	app->current_media = strdup(chan->url);
-	gui_send_play(app->gui,chan->url);
+    if ( app->current_media == NULL ) {        
+        app->current_media = strdup(chan->url);
+        gui_send_play(app->gui,chan->url);
     } else {
-	if ( app->next_media ) {
-	    free(app->next_media);
-	}
-	app->next_media = strdup(chan->url);
-	gui_send_message(app->gui,&rfb_media_stop);
+        if ( app->next_media ) {
+            free(app->next_media);
+        }
+        app->next_media = strdup(chan->url);
+        gui_send_message(app->gui,&rfb_media_stop);
     }
     gui_register_keys(app->gui,REGISTER_MEDIA,liveradio_keys,app);
     gui_register_ack(app->gui,REGISTER_MEDIA,liveradio_ack,app);
@@ -327,21 +369,21 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
     switch ( acktype ) {
     case RDC_STOP:     /* The client has sent a stop message */  
         if ( app->next_media ) {
-	    if ( app->current_media ) {
-		free(app->current_media);
-	    }
+            if ( app->current_media ) {
+                free(app->current_media);
+            }
             gui_send_play(app->gui,app->next_media);
             app->current_media = app->next_media;
             app->next_media = NULL;
         } else {
-	    /* Remove registered handlers */
-	    if ( app->current_media ) {
-		free(app->current_media);
-		app->current_media = NULL;
-	    }
-	    gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
-	    gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
-	}
+            /* Remove registered handlers */
+            if ( app->current_media ) {
+                free(app->current_media);
+                app->current_media = NULL;
+            }
+            gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
+            gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
+        }
         break;
     default:
         return 0;    /* Kick it up a level */
@@ -355,9 +397,10 @@ static int liveradio_ack(int acktype, void *param, unsigned char *buf, int len)
 
 
 
-static int app_recordings(menu_t *menu,app_t *app, int sel)
+static int app_recordings(menu_t *menu,void *iapp, int sel)
 {
-    menu_t   *child;
+    app_t     *app = iapp;
+    menu_t    *child;
     child = new_menu(app->dongle,app->render,menu,app,0);
     menu_set_title(child,"Video Recordings");  
 
@@ -366,23 +409,163 @@ static int app_recordings(menu_t *menu,app_t *app, int sel)
     return 1;
 }
 
-static int app_music(menu_t *menu,app_t *app, int sel)
+
+/*
+ *   Handle the root menu for Music menu
+ */
+static int app_music(menu_t *menu,void *iapp, int sel)
 {
-    menu_t   *child;
-    child = new_menu(app->dongle,app->render,menu,app,0);
+    app_t      *app = iapp;
+    dirlist_t  *dirlist;
+    menu_t     *child;
+    char        buf[256];
+    int         i;
+
+    dirlist = new_dirlist(app);
+
+    dirlist->path = strdup(c_mp3_dir);
+
+    /* Scan the dirlist */
+    dirlist->filelist = directory_scan(c_mp3_dir,&dirlist->filelist_num);
+
+    child = new_menu(app->dongle,app->render,menu,dirlist,0);
     menu_set_title(child,"Music");   
+    menu_set_app_clearup(child,delete_dirlist);
+
+    for ( i = 0; i < dirlist->filelist_num; i++ ) {
+        if ( dirlist->filelist[i].flags ) {
+            snprintf(buf,sizeof(buf),"[%s]",dirlist->filelist[i].name);
+        } else {
+            snprintf(buf,sizeof(buf),"%s",dirlist->filelist[i].name);
+        }       
+        menu_add(child,buf,process_music);
+    }
+
     menu_display(child);
     return 1;
 }
 
 
+static int process_music(menu_t *menu,void *iapp, int sel)
+{
+    char         buf[FILENAME_MAX+1];
+    dirlist_t    *dirlist = iapp;
+    dirlist_t    *dirlist2;
+    filelist_t   *entry;
+    menu_t       *child;
+    int           i;
+
+
+    entry = &dirlist->filelist[sel];
+
+    if ( entry->flags ) {
+        /* The selection was a directory - we need to create a new menu - hah! */
+        dirlist2 = new_dirlist(dirlist->app);
+
+        /* Scan the dirlist */
+        snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
+        dirlist2->path = strdup(buf);
+        dirlist2->filelist = directory_scan(buf,&dirlist2->filelist_num);
+
+        child = new_menu(dirlist2->app->dongle,dirlist2->app->render,menu,dirlist2,0);
+        snprintf(buf,sizeof(buf),"Music - %s",dirlist2->path);
+        menu_set_title(child,buf);   
+        menu_set_app_clearup(child,delete_dirlist);
+            
+        for ( i = 0; i < dirlist2->filelist_num; i++ ) {
+            if ( dirlist2->filelist[i].flags ) {
+                snprintf(buf,sizeof(buf),"[%s]",dirlist2->filelist[i].name);
+            } else {
+                snprintf(buf,sizeof(buf),"%s",dirlist2->filelist[i].name);
+            }       
+            menu_add(child,buf,process_music);
+        }
+        menu_display(child);
+        return 1;        
+    } else {
+	snprintf(buf,sizeof(buf),"%s/%s",dirlist->path,entry->name);
+        if ( dirlist->app->current_media == NULL ) {        
+            dirlist->app->current_media = strdup(buf);
+            gui_send_play(dirlist->app->gui,buf);
+        } else {
+            if ( dirlist->app->next_media ) {
+                free(dirlist->app->next_media);
+            }        
+            dirlist->app->next_media = strdup(buf);
+            gui_send_message(dirlist->app->gui,&rfb_media_stop);    
+        }
+        gui_register_keys(dirlist->app->gui,REGISTER_MEDIA,music_keys,dirlist->app);
+        gui_register_ack(dirlist->app->gui,REGISTER_MEDIA,music_ack,dirlist->app);
+    }  
+
+    return 1;     /* Carry on displaying this menu */
+}
+
+static int music_keys(void *ptr, int code)
+{
+    app_t     *app = ptr;
+
+    switch ( code ) { 
+    case kStop:
+        gui_send_message(app->gui,&rfb_media_stop);
+        menu_display(app->menu);
+        break;
+    case kVolUp:
+        gui_send_message(app->gui,&rfb_media_volup);
+        break;
+    case kVolDn:
+        gui_send_message(app->gui,&rfb_media_voldown);
+        break;
+    case kMute:
+        gui_send_message(app->gui,&rfb_media_mute);
+        break;
+    default:
+        return 0;      /* Carry on processing menu keys */
+    }   
+    return 1;
+}
+
+static int music_ack(int acktype, void *param, unsigned char *buf, int len)
+{
+    app_t         *app = param;
+
+    switch ( acktype ) {
+    case RDC_STOP:     /* The client has sent a stop message */  
+        if ( app->next_media ) {
+            if ( app->current_media ) {
+                free(app->current_media);
+            }
+            gui_send_play(app->gui,app->next_media);
+            app->current_media = app->next_media;
+            app->next_media = NULL;
+        } else {
+            /* Remove registered handlers */
+            if ( app->current_media ) {
+                free(app->current_media);
+                app->current_media = NULL;
+            }
+            gui_register_keys(app->gui,REGISTER_MEDIA,NULL,app);
+            gui_register_ack(app->gui,REGISTER_MEDIA,NULL,app);
+        }
+        break;
+    default:
+        return 0;    /* Kick it up a level */
+    }
+
+    return 1;        /* Don't process anymore */
+}
+
+
+
+
 /***********************
  *
- *   Handling of the Live TV Menus
+ *   Handling of the settings Menu
  *
  ***********************/
-static int app_settings(menu_t *menu,app_t *app, int sel)
+static int app_settings(menu_t *menu,void *iapp, int sel)
 {
+    app_t           *app = iapp;
     menu_t          *child;
     static char     *tvmode[]  = { "NTSC", "PAL" };
     static char     *flicker[] = { "None", "Low", "Medium", "High" };
@@ -400,10 +583,10 @@ static int app_settings(menu_t *menu,app_t *app, int sel)
 
     /* A little bit of fiddling - see rules in proces_settings */
     if ( app->dongle->videooutput != 0 ) {
-	menu_add_option(child,"Output:",process_settings,
-			MENU_INT,4,output,&app->dongle->videooutput);
+        menu_add_option(child,"Output:",process_settings,
+                        MENU_INT,4,output,&app->dongle->videooutput);
     } else {
-	menu_add(child,"Output: RGB Detected",process_settings);
+        menu_add(child,"Output: RGB Detected",process_settings);
     }
 
     menu_add_option(child,"Flicker Control:",process_settings,
@@ -419,9 +602,10 @@ static int app_settings(menu_t *menu,app_t *app, int sel)
     return 1;
 }
 
-static int process_settings(menu_t *menu,app_t *app, int sel)
+static int process_settings(menu_t *menu,void *iapp, int sel)
 {
-   /* Some rules preened from the configuration.html:
+    app_t   *app = iapp;
+    /* Some rules preened from the configuration.html:
 
     1). For NTSC unit -- in response to GUI query, video output value 
     is always '3', so you show "Video Output" as "Composite/Svideo" no changes
@@ -441,29 +625,131 @@ static int process_settings(menu_t *menu,app_t *app, int sel)
     /* If we come through with -1 then we should test the settings */
     switch ( sel ) {
     case -1:  /* If we come through with -1 then we should test the settings */
-	if ( app->dongle->tvmode == 0 ) {
-	    app->dongle->videooutput = 3;
-	}
+        if ( app->dongle->tvmode == 0 ) {
+            app->dongle->videooutput = 3;
+        }
         gui_send_settings(app->gui,RDC_SETTINGS_TEST);
         return 1;  /* Carry on displaying current menu */
     case 4:
-	gui_send_settings(app->gui,RDC_SETTINGS_CANCEL);
-	return 0;
+        gui_send_settings(app->gui,RDC_SETTINGS_CANCEL);
+        return 0;
     case 5:
-	gui_send_settings(app->gui,RDC_SETTINGS_SAVE);
-	return 0;
+        gui_send_settings(app->gui,RDC_SETTINGS_SAVE);
+        return 0;
     default:
-	return 0;   /* Carry on displaying */
+        return 0;   /* Carry on displaying */
     } 
 }
 
 
 
+static dirlist_t    *new_dirlist(app_t *app)
+{
+    dirlist_t   *dir = malloc(sizeof(*dir));
+
+    dir->filelist_num = 0;
+    dir->filelist = NULL;
+    dir->app = app;
+
+    return dir;
+}
+
+static void          delete_dirlist(void *idir)
+{
+    dirlist_t *dir = idir;
+    int        i;
+
+    for ( i = 0; i < dir->filelist_num; i++ ) {
+        free(dir->filelist[i].name);
+    }
+    if ( dir->filelist ) {
+        free(dir->filelist);
+    }
+
+    free(dir->path);
+
+    free(dir);
+}
+
+static filelist_t *directory_scan(char *path, int *scanned)
+{
+    DIR    *dir;
+    struct dirent *dirent;
+    struct stat sb;
+    char   filename[FILENAME_MAX+1];
+    filelist_t *list = NULL;
+    int         num  = 0;
+    int         flags;
+    int         i;
+
+
+    *scanned = 0;
+
+    if ( ( dir = opendir(path) ) == NULL ) {
+        perror("opendir");
+        return NULL;
+    }
+
+    while ( (dirent = readdir(dir)) != NULL ) {
+        if ( dirent->d_name[0] == '.' ) {
+            continue;
+        }
+
+        snprintf(filename,sizeof(filename),"%s/%s",path,dirent->d_name);
+        if ( stat(filename,&sb) < 0 ) {
+            continue;
+        }
+
+        if ( S_ISDIR(sb.st_mode) ) {
+            flags = 1;
+        } else {
+            flags = 0;
+        }
+
+        i = num++;
+        list = realloc(list, num * sizeof(filelist_t));
+
+        list[i].name = strdup(dirent->d_name);
+        list[i].flags = flags;
+            
+    }
+
+    closedir(dir);
+
+    directory_sort(num,list);
+
+    *scanned = num;
+    return list;
+}
+
+static void directory_sort(int num,filelist_t *list)
+{
+    int    i,j;
+    int    t_flags;
+    char  *t_name;
+
+    for ( i = (num-1) ; i >= 0; i-- ) {
+        t_name = list[i].name;
+        t_flags = list[i].flags;
+        j = i + 1;
+
+        while ( j < num && strcasecmp(list[j].name,t_name) < 0 ) {
+            list[j-1].name = list[j].name;
+            list[j-1].flags = list[j].flags;
+
+            j++;
+        }
+
+        list[j-1].name = t_name;
+        list[j-1].flags = t_flags;
+
+    }
+}
 
 
 
 static int read_channel_file(char *filename, int *num, channel_t **list, 
-			     time_t *update_time)
+                             time_t *update_time)
 {
     char         buf[500];
     int          channum;
@@ -509,7 +795,7 @@ static int read_channel_file(char *filename, int *num, channel_t **list,
 
             chan = realloc(chan,channum * sizeof(channel_t));
             chan[i].name = strdup(buf);
-            chan[i].url = strdup(ptr);	    
+            chan[i].url = strdup(ptr);            
         }
         *list = chan;
         *num = channum;
