@@ -2,8 +2,8 @@
  *  Header for MediaMVP server
  *
  *
- *  $Id: mediamvp.h,v 1.22 2003/11/24 21:52:00 dom Exp $
- *  $Date: 2003/11/24 21:52:00 $
+ *  $Id: mediamvp.h,v 1.23 2003/11/25 17:25:44 dom Exp $
+ *  $Date: 2003/11/25 17:25:44 $
  */
 
 #ifndef _MEDIAMVP_H_
@@ -112,18 +112,18 @@ extern int             http_open(char *url);
 
 /* menu.c */
 #define MENU_NUMBERS   1
-extern menu_t         *new_menu(dongle_t *dongle, render_t *render, menu_t *parent,app_t *app, int flags);
+extern menu_t         *new_menu(dongle_t *dongle, render_t *render, menu_t *parent,void *app, int flags);
 extern void            delete_menu_tree(menu_t *menu);
 extern void            delete_menu(menu_t *menu);
 extern void            menu_set_title(menu_t *menu, char *text);
 #define   MENU_NONE    0
 #define   MENU_INT     1
 extern void            menu_add_option(menu_t *menu, char *name, 
-                                       int (callback_func)(menu_t *menu,app_t *app, int sel),
+                                       int (callback_func)(menu_t *menu,void *app, int sel),
                                        int type, int num_options, char **options, void *value_ptr);
-extern void            menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel));
+extern void            menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,void *app, int sel));
 extern void            menu_display(menu_t *menu);
-
+extern void            menu_set_app_clearup(menu_t *menu, void (*clearup)(void *ptr));
 
 
 /* misc.c */
