@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: menu.c,v 1.14 2003/11/24 21:52:00 dom Exp $
- *   $Date: 2003/11/24 21:52:00 $
+ *   $Id: menu.c,v 1.19 2003/11/27 20:25:13 dom Exp $
+ *   $Date: 2003/11/27 20:25:13 $
  *
  *
  *   Constructs a menu and all such things
@@ -29,7 +29,7 @@
 
 typedef struct {
     char    *name;
-    int    (*callback_func)(menu_t *menu,app_t *app, int sel);
+    int    (*callback_func)(menu_t *menu,void *app, int sel);
     int      num_options;
     char   **options;
     int      type;
@@ -40,7 +40,7 @@ typedef struct {
 struct _menu {
     dongle_t     *dongle;
     render_t     *render;
-    app_t        *app;
+    void         *app;
     gui_t        *gui;
     int           numoptions;
     option_t     *options;
@@ -49,15 +49,21 @@ struct _menu {
     int           current_selection;
     int           maxrows;                /*!< Number of lines that can be displayed on screen */
     int           flags;
+    void         (*app_clearup)(void *ptr);
     menu_t        *parent;
     menu_t        *child;
+    char          *red;
+    char          *green;
+    char          *yellow;
+    char          *blue;
+    int          (*colour_func)(menu_t *menu,void *app, int sel);
 };
 
 static int        menu_keys(void *ptr, int code);
 
 
 
-menu_t *new_menu(dongle_t *dongle,render_t *render, menu_t *parent, app_t *app, int flags)
+menu_t *new_menu(dongle_t *dongle,render_t *render, menu_t *parent, void *app, int flags)
 {
     menu_t       *menu = malloc(sizeof(*menu));
 
@@ -71,7 +77,9 @@ menu_t *new_menu(dongle_t *dongle,render_t *render, menu_t *parent, app_t *app,
     menu->options = NULL;
     menu->current_selection = 0;
     menu->flags   = flags;
+    menu->app_clearup = NULL;
     menu->maxrows = render_get_maxrows(render);
+    menu->red = menu->green = menu->blue = menu->yellow = NULL;
 
     if ( parent ) {
         menu->parent = parent;
@@ -89,13 +97,13 @@ void delete_menu_tree(menu_t *menu)
     menu_t   *temp;
 
     while ( menu->child ) {
-	menu = menu->child;
+        menu = menu->child;
     }
 
     while ( menu->parent ) {
-	temp = menu->parent;
-	delete_menu(temp);
-	menu = temp;
+        temp = menu->parent;
+        delete_menu(temp);
+        menu = temp;
     }
 
 }
@@ -115,6 +123,29 @@ void delete_menu(menu_t *menu)
     if ( menu->options ) {
         free(menu->options);
     }
+
+    if ( menu->app_clearup ) {
+        (menu->app_clearup)(menu->app);
+    }
+
+    if ( menu->red ) {
+        free(menu->red);
+    }
+
+    if ( menu->green ) {
+        free(menu->green);
+    }
+
+    if ( menu->yellow ) {
+        free(menu->yellow);
+    }
+
+    if ( menu->blue ) {
+        free(menu->blue);
+    }
+
+
+    free(menu);
 }
 
 void menu_set_title(menu_t *menu, char *text)
@@ -127,7 +158,7 @@ void menu_set_title(menu_t *menu, char *text)
 }
 
 
-void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,app_t *app, int sel),
+void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,void *app, int sel),
                      int type, int num_options, char **options, void *value_ptr)
 {
     int     i = menu->numoptions++;
@@ -142,7 +173,7 @@ void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,
     menu->options[i].value_ptr = value_ptr;
 }
 
-void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel))
+void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,void *app, int sel))
 {
     int     i = menu->numoptions++;
 
@@ -185,7 +216,7 @@ void menu_display(menu_t *menu)
     last = (menu->top_offset + (menu->maxrows - 2) < menu->numoptions) ? menu->top_offset + (menu->maxrows - 2) : menu->numoptions;
     for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
         if ( menu->options[i].type == MENU_INT ) {
-	    printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
+            printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
             extra = menu->options[i].options[*((int *)menu->options[i].value_ptr)];
         } else {
             extra = "";
@@ -213,14 +244,14 @@ static int menu_keys(void *ptr, int code)
 {
     menu_t    *menu = ptr;
     int        pos;
-    int      (*func)(menu_t *,app_t *app, int sel);
+    int      (*func)(menu_t *,void *app, int sel);
     option_t  *option;
     unsigned int       *num;
     int      streamtype = 0;
 
 
     if ( menu->dongle->stream ) {
-	streamtype = stream_get_type(menu->dongle->stream);
+        streamtype = stream_get_type(menu->dongle->stream);
     }
 
     printf("Got menu key %d\n",code);
@@ -229,115 +260,115 @@ static int menu_keys(void *ptr, int code)
        without going back to the menu 
     */
     if ( streamtype == ( MEDIA_MPEG | MEDIA_SOCKET ) ) {
-	switch ( code ) {
-	case kChanUp:
-	    if ( menu->current_selection < menu->numoptions - 1 ) {
-		menu->current_selection++;
-		if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
-		    menu->top_offset++;
-		}
-	    } else {
-		menu->current_selection = 0;
-		menu->top_offset = 0;
-	    }
+        switch ( code ) {
+        case kChanUp:
+            if ( menu->current_selection < menu->numoptions - 1 ) {
+                menu->current_selection++;
+                if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
+                    menu->top_offset++;
+                }
+            } else {
+                menu->current_selection = 0;
+                menu->top_offset = 0;
+            }
 
-	    func = menu->options[menu->current_selection].callback_func;
-	    if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
-		menu_t  *parent;		
-		parent = menu->parent;
-		delete_menu(menu);
-		parent->child = NULL;	
-	    }  
-	    break;
-	case kChanDn:
-	    if ( menu->current_selection > 0 ) {
-		menu->current_selection--;
-		if ( menu->top_offset > menu->current_selection ) {
-		    menu->top_offset = menu->current_selection;
-		}
-	    } else {
-		menu->current_selection = menu->numoptions - 1;	  
-		if ( menu->numoptions > menu->maxrows - 2 ) {
-		    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
-		}
-	    }
-	    func = menu->options[menu->current_selection].callback_func;
-	    if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
-		menu_t  *parent;		
-		parent = menu->parent;
-		delete_menu(menu);
-		parent->child = NULL;	
-	    }  
-	    break;
-	}
-	return 0;
+            func = menu->options[menu->current_selection].callback_func;
+            if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
+                menu_t  *parent;                
+                parent = menu->parent;
+                delete_menu(menu);
+                parent->child = NULL;        
+            }  
+            break;
+        case kChanDn:
+            if ( menu->current_selection > 0 ) {
+                menu->current_selection--;
+                if ( menu->top_offset > menu->current_selection ) {
+                    menu->top_offset = menu->current_selection;
+                }
+            } else {
+                menu->current_selection = menu->numoptions - 1;          
+                if ( menu->numoptions > menu->maxrows - 2 ) {
+                    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
+                }
+            }
+            func = menu->options[menu->current_selection].callback_func;
+            if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
+                menu_t  *parent;                
+                parent = menu->parent;
+                delete_menu(menu);
+                parent->child = NULL;        
+            }  
+            break;
+        }
+        return 0;
     }
 
 
 
     switch ( code ) {
     case kChanUp:
-	if ( streamtype != MEDIA_MPEG ) {
-	    if ( menu->current_selection > 0 ) {
-		menu->current_selection--;
-		if ( menu->top_offset > menu->current_selection ) {
-		    menu->top_offset = menu->current_selection;
-		}
-	    } else {
-		menu->current_selection = menu->numoptions - 1;	  
-		if ( menu->numoptions > menu->maxrows - 2 ) {
-		    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
-		}
-	    }
-	    
-	    menu_display(menu);
-	}
+        if ( streamtype != MEDIA_MPEG ) {
+            if ( menu->current_selection > 0 ) {
+                menu->current_selection--;
+                if ( menu->top_offset > menu->current_selection ) {
+                    menu->top_offset = menu->current_selection;
+                }
+            } else {
+                menu->current_selection = menu->numoptions - 1;          
+                if ( menu->numoptions > menu->maxrows - 2 ) {
+                    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
+                }
+            }
+            
+            menu_display(menu);
+        }
         break;
     case kChanDn:
-	if ( streamtype != MEDIA_MPEG ) {
-	    if ( menu->current_selection < menu->numoptions - 1 ) {
-		menu->current_selection++;
-		if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
-		    menu->top_offset++;
-		}
-	    } else {
-		menu->current_selection = 0;
-		menu->top_offset = 0;
-	    }
+        if ( streamtype != MEDIA_MPEG ) {
+            if ( menu->current_selection < menu->numoptions - 1 ) {
+                menu->current_selection++;
+                if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
+                    menu->top_offset++;
+                }
+            } else {
+                menu->current_selection = 0;
+                menu->top_offset = 0;
+            }
 
-	    if ( streamtype == ( MEDIA_MPEG | MEDIA_SOCKET ) ) {
-		menu_t  *parent;
-		func = menu->options[menu->current_selection].callback_func;
-		if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
-		    parent = menu->parent;
-		    delete_menu(menu);
-		    parent->child = NULL;	
-		    menu_display(parent);
-		}  
-	    } else if ( (streamtype & MEDIA_MASK) != MEDIA_MPEG ) {
-		menu_display(menu);
-	    }
-	}
+            if ( streamtype == ( MEDIA_MPEG | MEDIA_SOCKET ) ) {
+                menu_t  *parent;
+                func = menu->options[menu->current_selection].callback_func;
+                if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
+                    parent = menu->parent;
+                    delete_menu(menu);
+                    parent->child = NULL;        
+                    menu_display(parent);
+                }  
+            } else if ( (streamtype & MEDIA_MASK) != MEDIA_MPEG ) {
+                menu_display(menu);
+            }
+        }
         break;
     case kVolDn:
         option = &menu->options[menu->current_selection];
         if ( option->type == MENU_INT ) {
-	    menu_t *parent = menu->parent;
+            menu_t *parent = menu->parent;
             num = option->value_ptr;           
             *num = ( *num + 1 ) % option->num_options;
             func = menu->options[menu->current_selection].callback_func;
             if ( func && func(menu,menu->app,-1) == 0 ) {
                 delete_menu(menu);
-                parent->child = NULL;	
+                parent->child = NULL;        
             }    
-	    menu_display(parent);
+            menu_display(parent);
   
         }
         break;
     case kVolUp:
         option = &menu->options[menu->current_selection];
         if ( option->type == MENU_INT ) {
-	    menu_t *parent = menu->parent;
+            menu_t *parent = menu->parent;
             num = option->value_ptr;
             if ( *num == 0 ) {
                 *num = option->num_options - 1;
@@ -347,9 +378,9 @@ static int menu_keys(void *ptr, int code)
             func = menu->options[menu->current_selection].callback_func;
             if ( func && func(menu,menu->app,-1) == 0 ) {
                 delete_menu(menu);
-                parent->child = NULL;	
+                parent->child = NULL;        
             }      
-	    menu_display(parent);
+            menu_display(parent);
 
         }
         break;
@@ -377,7 +408,7 @@ static int menu_keys(void *ptr, int code)
             if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
                 parent = menu->parent;
                 delete_menu(menu);
-                parent->child = NULL;	
+                parent->child = NULL;        
                 menu_display(parent);
             }        
         }
@@ -393,6 +424,16 @@ static int menu_keys(void *ptr, int code)
             }
         }
         break;
+    case kRed ... kBlue:
+        func = menu->colour_func;        
+        if ( func && func(menu,menu->app,code - kRed) == 0 ) {
+	    menu_t *parent;
+            parent = menu->parent;
+            delete_menu(menu);
+            parent->child = NULL;        
+            menu_display(parent);
+        }     
+        break;
     default:
         printf("Received IR code %d - not used here\n",code);
     }
@@ -400,6 +441,63 @@ static int menu_keys(void *ptr, int code)
 }
 
 
+/** \brief Set the clearup pointer for the app datatype 
+ * 
+ *  \param menu Menu handle
+ *  \param clearup Function to call to clearup
+ *
+ *  Used for those situations when we recurse down through directory structures
+ */
+void menu_set_app_clearup(menu_t *menu, void (*clearup)(void *ptr))
+{
+    menu->app_clearup = clearup;
+}
+
+
+/** \brief Set what the colour keys do 
+ *
+ *  \param menu Menu handle
+ *  \param red     Red button text
+ *  \param green   Green button text
+ *  \param yellow  Yellow button text
+ *  \param blue    Blue button text
+ *  \param func    Callback function - called back with 0 - 3 for the colour button that has been
+ *                 pressed
+ */
+void menu_set_colactions(menu_t *menu,
+                         char   *red,
+                         char   *green,
+                         char   *yellow,
+                         char   *blue,
+                         int   (*func)(menu_t *menu,void *app, int sel))
+{
+
+    if ( menu->red ) {
+        free(menu->red);
+    }
+    menu->red = strdup(red);
+
+    if ( menu->green ) {
+        free(menu->green);
+    }
+    menu->green = strdup(green);
+
+    if ( menu->yellow ) {
+        free(menu->yellow);
+    }
+    menu->yellow = strdup(yellow);
+
+    if ( menu->blue ) {
+        free(menu->blue);
+    }
+    menu->blue = strdup(blue);
+
+    menu->colour_func = func;
+}
+
+
+
+
  
 /*
  * Local Variables:
